#include "math.h"
const double m_ln2=0.69314718055994530942;
const double m_twopi=6.2831853071795865;
const double m_sqrt3=1.73205080756887729;

const double m_undef=0.0/0.0;
const double m_plusinf=1.0/0.0;
const double m_minusinf=-1.0/0.0;

  // 14!(14-k)!
  double fact14[]={1.0,14.0,182.0,2184.0,24024.0,240240.0,2162160.0,17297280.0,121080960.0,726485760.0,3632428800.0,14529715200.0,43589145600.0,87178291200.0};

  // k is an integer >=0 and < 1024
  double exp2(double k){
    if (k<31.0){
      unsigned i=0x1 << (int) k;
      return i;
    }
    double k1=(int)(k/32.0+0.001); // k1<32
    double res=1.0;
    k -= k1*32.0; // k<32
    for (;k>0.0;k -=1.0)
      res *= 2.0;
    for (;k1>0.0; k1 -= 1.0)
      res *= 4294967296.0;
    return res;
  }

  double exp(double d){
    if (d<0)
      return 1.0/exp(-d);
#ifdef DEBUG
    ofstream of("int.txt");
#endif
    double k=(int)(d/m_ln2+.5);
#ifdef DEBUG
    of << "k:" << k << char(10) << char(13) ;
#endif
    if (k>1023.0)
      return m_plusinf;
    d=d-k*m_ln2; // |d|<m_ln2/2
#ifdef DEBUG
    of << "d:" << d << char(10) << char(13) ;
#endif
    if (k<0)
      k=1.0/exp2(-k);
    else
      k=exp2(k);
#ifdef DEBUG
    of << "2^k:" << k << char(10) << char(13) ;
#endif
    // Insure d is < 0, if it was > 0 compute inv(exp(-d))
    int inv=d>0;
    if (inv)
      d=-d;
    // use Taylor expansion at order 14 since
    // (1+(ln(2.0)/2)^14/14!)-1 -> 0.0
    double res=d;
    for (int i=1;i<14;++i){
      res += fact14[i];
      res *= d;
#ifdef DEBUG
      of << "i:" << i << " ,res:" << res << char(10) << char(13) ;
#endif
    }
    res /= 87178291200.0;
    res += 1.0;
#ifdef DEBUG
    of << "res:" << res << char(10) << char(13) ;
#endif
    if (inv)
      return k/res;
    else
      return res*k;
  }

  double sinh(double d){
    double k=exp(d);
    return k-1.0/k;
  }

  double cosh(double d){
    double k=exp(d);
    return k+1.0/k;
  }

  double tanh(double d){
    if (d>700.0)
      return 1.0;
    if (d<-700.0)
      return -1.0;
    double k=exp(d);
    k=k*k;
    return (k-1)/(k+1);
  }

  // 0<d<=pi/4
  double in_tan(double d){
    if (d>0.07){
      // tan(2*x)=2*tan(x)/(1-tan(x)^2)
      double tandsur2=in_tan(d/2);
      return 2.0*tandsur2/(1.0-tandsur2*tandsur2);
    }
    // d*(1382*d^2^5+3410*d^2^4+8415*d^2^3+20790*d^2^2+51975*d^2+155925)/155925
    double d2=d*d;
    double res=((((1382.0*d2+3410.0)*d2+8415.0)*d2+20790.0)*d2+51975.0)*d2+155925.0;
    return res*d/155925.0;
  }

  double tan(double d){
    if (d<0)
      return -tan(-d);
    double k=(int)(d/M_PI+.5); 
    d=d-k*M_PI; // |d|<pi/2
    int neg=d<0,inv=0;
    if (neg)
      d=-d;
    if (d>M_PI_4){
      d=M_PI_2-d;
      inv=1;
    }
    // 0<d<=pi/4
    k= in_tan(d);
    if (neg)
      k=-k;
    if (inv)
      k=1.0/k;
    return k;
  }

  double sin(double d){
    if (d<0)
      return -sin(-d);
    double k=(int)(d/m_twopi+.5); 
    d=d-k*m_twopi; 
    int neg=d<0;
    if (neg)
      d=-d;
    if (d>M_PI_2)
      d=M_PI-d;
    // now |d|<pi/2
    k=in_tan(d/2.0);
    k=(2.0*k)/(k*k+1.0);
    if (neg)
      k=-k;
    return k;
  }

double fmod(double x,double y){
  return x-x*(int)(y/x);
}

  double cos(double d){
    if (d<0)
      return cos(-d);
    double k=(int)(d/m_twopi+.5); 
    d=d-k*m_twopi; 
    if (d<0)
      d=-d;
    int neg=0;
    if (d>M_PI_2){
      neg=1;
      d=M_PI-d;
    }
    // now |d|<pi/2, compute sin(pi/2-d)
    k=in_tan((M_PI_2-d)/2.0);
    k=(2.0*k)/(k*k+1.0);
    if (neg)
      k=-k;
    return k;
  }

  double floor(double d){
    if (d>0)
      return (int)(d);
    double k=(int)(d);
    if (k==d)
      return k;
    else
      return k-1;
  }

  double ceil(double d){
    if (d<0)
      return (int)(d);
    double k=(int)(d);
    if (k==d)
      return k;
    else
      return k+1;
  }

int rand(){
  static unsigned r=0;
  r = (unsigned) ((1664525*(unsigned long long)(r)+1013904223)%((unsigned)(1)<<31));
  return r;
}

  // Taylor expansion order 23, |d|<0.27
  double in_atan(double d){
    double d2=d*d;
    double res=((((((((((-14549535.0*d2+15935205.0)*d2-17612595.0)*d2+19684665.0)*d2-22309287.0)*d2+25741485.0)*d2-30421755.0)*d2+37182145.0)*d2-47805615.0)*d2+66927861.0)*d2-111546435.0)*d2+334639305.0;
    return res*d/334639305.0;
  }

  double atan(double d){
    if (d<0)
      return -atan(-d);
    int inv=0;
    if (d>1){
      inv=1;
      d=1.0/d;
    }
    double k;
    if (d<0.265)
      k=in_atan(d);
    else {
      if (d>0.7)
	k=M_PI_4-in_atan((1.0-d)/(1.0+d));
      else
	k=M_PI/6.0+in_atan((m_sqrt3*d-1)/(m_sqrt3+d));
    }
    if (inv)
      k=M_PI_2-k;
    return k;
  }

  double pow2tab[]={0.13407807929942597100e155,0.11579208923731619542e78,0.34028236692093846346e39,0.18446744073709551616e20,0.42949672960000000000e10,65536.0,256.0,16.0,4.0,2.0,1.4142135623730950488,1.1892071150027210667,1.0905077326652576592};
  double sqrt(double d){
    if (d<0)
      return m_undef;
    if (d==0 || d==1)
      return d;
    if (d<1)
      return 1.0/sqrt(1.0/d);
    double k=0.0,k2=512.0;
    for (int i=0;i<8;++i,k2 /= 2.0){
      if (d>=pow2tab[i]){
	k += k2;
	d /= pow2tab[i];
      }
    }
    double d0=(1.0+d)/2.0,d1;
    for (;;d0=d1){
      d1=(d0+d/d0)/2.0;
      if ((d0-d1)<1e-15)
	return d1*exp2(k/2.0);
    }
  }

  double asin(double d){
    if (d==1 || d==-1)
      return d*M_PI_2;
    double d2=d*d;
    if (d2>1)
      return m_undef;
    return atan(d/sqrt(1-d2));
  }

  double acos(double d){
    return M_PI_2-asin(d);
  }

  double log(double d){
    if (d<0)
      return m_undef;
    if (d==0)
      return m_minusinf;
    if (d<1)
      return -log(1.0/d);
    double k=0.0,k2=512.0; 
    // find number of powers of 2 in d
    for (int i=0;i<12;++i,k2 /= 2.0){
      if (d>=pow2tab[i]){
	k += k2;
	d /= pow2tab[i];
      }
    }
    // now d>1 and d<2^(1/8) -> d-1>0 and ||<0.091
    // Taylor expansion at order 14
    d -= 1;
    double res=(((((((((((((-25740.0*d+27720.0)*d-30030.0)*d+32760.0)*d-36036.0)*d+40040.0)*d-45045.0)*d+51480.0)*d-60060.0)*d+72072.0)*d-90090.0)*d+120120.0)*d-180180.0)*d+360360.0)*d;
    return res/360360.0+k*m_ln2;
  }

  double log10(double d){
    return log(d)/M_LN10;
  }

  double acosh(double d){
    double d2=d*d;
    if (d2<1)
      return m_undef;
    return log(d+sqrt(d2-1));
  }

  double asinh(double d){
    double d2=d*d;
    return log(d+sqrt(d2+1));
  }

  double atanh(double d){
    if (d==1)
      return m_plusinf;
    if (d==-1)
      return m_minusinf;
    double d2=d*d;
    if (d2>1)
      return m_undef;
    return log(sqrt(1-d2)/(1-d));
  }

  double pow(double x,double y){
    return exp(y*log(x));
  }

  double hypot(double x,double y){
    return sqrt(x*x+y*y);
  }

  double atan2(double x,double y){
    if (x==0){
      if (y>0)
	return M_PI_2;
      if (y<0)
	return -M_PI_2;
      return m_undef;
    }
    double res=atan(y/x);
    if (x>0)
      return res;
    if (y>0)
      return M_PI+res;
    else
      return res-M_PI;
  }

double fabs(double x){
  return x<0?-x:x;
}
